% This program replicates the empirical results in
% Djogbenou, A. A. (2023)
% Results incorporating Sign Restrictions and Inventories
% Last modified: September 2023
% Produce Figures C22, C23, C24 and C25 in Online Appendix C

clear all;
clc;

% Data
[oil,developed,emerging,km]=data_import_inventory();

T = size(km,1);
[T0,q0] = size(oil);  % The first variable is oil supply and the second is oil price over T periods
[T0,N1] = size(developed); % Developed economy real activity variables 
[T0,N2] = size(emerging); % Emerging economy real activity variables 

oil(T+1:T0,:)=[];
developed(T+1:T0,:)=[];
emerging(T+1:T0,:)=[];

D=[0,0,1,0;0,0,0,1];
for jj=1:(T-2-2)/4 
    D=[D; [1,0,0,0;0,1,0,0;0,0,1,0;0,0,0,1]];
end
 D=[D; [1,0,0,0;0,1,0,0]];
km1=(eye(T)-D*inv(D'*D)*D')*km; % Seasonally ajusted inventories

G0  = (oil-repmat(mean(oil,1),T,1))./repmat(std(oil,1),T,1);
X1 = (developed-repmat(mean(developed,1),T,1))./repmat(std(developed,1),T,1);
X2 = (emerging-repmat(mean(emerging,1),T,1))./repmat(std(emerging,1),T,1);
G  =[G0(:,1) (km1-repmat(mean(km1,1),T,1))./repmat(std(km1,1),T,1) G0(:,2)];
q0=q0+1;
X = [X1 X2];
[T,N] = size(X);

% Parameter values
H=10;       % Maximum forecast horizon
NB=999;     % Number of bootstrap replications
alpha1=0.32;% 1-alpha1 bootstrap interval for impulse responce
r0=1;       % Number of global economy real activity factors
r01=1;      % Number of developed economy real activity factors
r02=1;      % Number of emerging economy real activity factors
p=4;       % Number of lags in the FAVAR model
r=r0+r01+r02; %Number of unobserved factors
q=q0;          %Number of unobserved factors
K=r+q;        % Number of factors in the FAVAR model



% ===============Model Estimation =========================================


%Filter out the observed factors
Gamma_tilde = (X'*G)/(G'*G);
Y = X- G*Gamma_tilde'; 

%Preliminary estimates for latent factors and loadings 
[Lambda_tilde,F_tilde,tol,count]=spce(Y,N1,r0,r01,r02);

% Check signs
C1=20;  % US GDP
C2=39+4;  % China GDP
     if Lambda_tilde(C1,1)<0  
        Lambda_tilde(:,1)=-Lambda_tilde(:,1);
        F_tilde(:,1)=-F_tilde(:,1);
      else
        Lambda_tilde(:,1)=Lambda_tilde(:,1);
        F_tilde(:,1)=F_tilde(:,1);
     end
     if Lambda_tilde(C1,2)<0  
        Lambda_tilde(:,2)=-Lambda_tilde(:,2);
        F_tilde(:,2)=-F_tilde(:,2);
      else
        Lambda_tilde(:,2)=Lambda_tilde(:,2);
        F_tilde(:,2)=F_tilde(:,2);
     end

     if Lambda_tilde(C2,3)<0  
        Lambda_tilde(:,3)=-Lambda_tilde(:,3);
        F_tilde(:,3)=-F_tilde(:,3);
      else
        Lambda_tilde(:,3)=Lambda_tilde(:,3);
        F_tilde(:,3)=F_tilde(:,3);
     end


Z_tilde=[F_tilde,G];
Delta_tilde=[Lambda_tilde,Gamma_tilde];

% Preliminary FAVAR Coefficient Estimates 
[B0,Omega,epsilon]=favar_estimation(Z_tilde,p);
B_tilde= B0;

%Updated FAVAR Coefficients
Delta123_tilde=-Omega(1:3,4:6)*inv(Omega(4:6,4:6));

Delta1_tilde=Delta123_tilde(:,1);
Delta2_tilde=Delta123_tilde(:,2);
Delta3_tilde=Delta123_tilde(:,3);

R_tilde=[eye(r), Delta1_tilde, Delta2_tilde,  Delta3_tilde;
           zeros(1,r), 1, 0, 0;
           zeros(1,r), 0, 1, 0;
           zeros(1,r), 0, 0, 1];

B_hat=zeros(p*K,K);
for j=1:p
B_hat((j-1)*K+1:j*K,:)=inv(R_tilde')*B_tilde((j-1)*K+1:j*K,:)*R_tilde';
end


%Final estimates for observed and unobserved factor loadings 
Lambda_hat=Lambda_tilde;
Gamma_hat(:,1)=Gamma_tilde(:,1)-Lambda_tilde*Delta1_tilde;
Gamma_hat(:,2)=Gamma_tilde(:,2)-Lambda_tilde*Delta2_tilde;
Gamma_hat(:,3)=Gamma_tilde(:,3)-Lambda_tilde*Delta3_tilde;
Delta_hat= [Lambda_hat Gamma_hat];

%Final estimates for unobserved factors 
F_hat=F_tilde+G(:,1)*Delta1_tilde'+G(:,2)*Delta2_tilde'+G(:,3)*Delta3_tilde';
Z_hat=[F_hat G];

%=================== Computation of Impulse response function ========================
IRF_Z    = zeros(H+1,K,K); % for unobserved and observed factors
IRF_Z_lower    = zeros(H+1,K,K);
IRF_Z_upper    = zeros(H+1,K,K);
IRF_Z1    = zeros(H+1,K,K); % for unobserved and observed factors
IRF_Z_lower1    = zeros(H+1,K,K);
IRF_Z_upper1    = zeros(H+1,K,K);

epsilon_centered=epsilon-repmat(mean(epsilon,1),size(epsilon,1),1);

e=X-Z_tilde*Delta_tilde';
e_centered=e-repmat(mean(e,1),size(e,1),1);

for shock=1:K
 tamp=zeros(1,K);
 tamp(1,shock) = 1;
 
 [IRF_Z(:,:,shock),IRF_Z_lower(:,:,shock),IRF_Z_upper(:,:,shock)] = favar_irf_median_response_inventory(Z_tilde,H,p,K,tamp);% IRF:  H+1 x K
end

% Normalization

    for shock = 1 : K
        if shock == 4
            scale = -1/IRF_Z(1,shock,shock);
        else
            scale =  1/IRF_Z(1,shock,shock);
        end   
      for k = 1 : K
        IRF_Z1(:,k,shock)=squeeze(IRF_Z(:,k,shock))*scale;

        IRF_Z_lower1(:,k,shock)=squeeze(IRF_Z_lower(:,k,shock))*scale;
        IRF_Z_upper1(:,k,shock)=squeeze(IRF_Z_upper(:,k,shock))*scale;
      end

    end

%======================Plot of IRFs ====================================
% Plot Figures C22, C23, C24 and C25 in Online Appendix C
plot_IRF_Z_with_inventory(IRF_Z1,IRF_Z_lower1,IRF_Z_upper1,H);

